module ClientThread (initClientThread) where

import Control.Concurrent
import Control.Concurrent.Chan
import Control.Monad
import Control.Monad.Reader
import Network
import System.IO

import Prot

type ClientM = ReaderT ClientState IO

data ClientState = ClientState {
    handle  :: Handle,
    inChan  :: Chan ServerMessage,
    outChan :: Chan ClientMessage
  }

recvClient :: ClientM (Maybe ClientMessage)
recvClient = do
  hndl <- asks handle
  liftIO $ catch
    (maybeRead `fmap` hGetLine hndl)
    (\_ -> return Nothing)
  where

    maybeRead :: Read a => String -> Maybe a
    maybeRead str = case reads str of
      [(x, "")] -> Just x
      _ -> Nothing


sendClient :: ServerMessage -> ClientM ()
sendClient msg = do
  hndl <- asks handle
  liftIO $ catch
    (hPutStrLn hndl (show msg))
    (\_ -> return ())

sendServer :: ClientMessage -> ClientM ()
sendServer msg = do
  chan <- asks outChan
  liftIO $ writeChan chan msg

recvServer :: ClientM ServerMessage
recvServer = do
  chan <- asks inChan
  liftIO $ readChan chan

clientThread :: ClientM ()
clientThread = do
  tryClientToServer
  continue <- tryServerToClient
  when continue $ do
    clientThread

tryClientToServer :: ClientM ()
tryClientToServer = do
  hndl <- asks handle
  isReady <- liftIO (hReady hndl)
  when isReady $ do
    msg <- recvClient
    case msg of
      Nothing -> return ()
      Just msg -> sendServer msg
  
tryServerToClient :: ClientM Bool
tryServerToClient = do
  chan <- asks inChan
  isEmpty <- liftIO (isEmptyChan chan)
  if isEmpty
    then return True
    else do
      msg <- recvServer
      sendClient msg
      case msg of
	End _ -> return False
	_   -> return True

initClientThread :: Handle -> Chan ClientMessage -> Chan ServerMessage -> IO ()
initClientThread hndl cchan schan = do
  hSetBuffering hndl LineBuffering
  runReaderT clientThread (ClientState hndl schan cchan)
  hClose hndl
