module Prot 
  where

----
-- Kliendi nimi
type Name = String

----
-- Kliendi poolt genereeritud miinivälja esitus.
-- Iga tabeli väärtus peab kas olema 1 või 0 mis tähistavad
-- vastavalt kas antud positsioonis asub miin või mitte.
type Board = [[Int]]

----
-- Koordinaat.
-- NB! Esimene komponent määrab tabeli rea (kõrgused) ning teine veeru
-- (laiuse).
-- Ülemine vasak = (1, 1)
-- Parem alumine = (height, width)
type Coord = (Int, Int)

----
-- Avatud koordinaadi esitus. 
-- Peale positsiooni teame nüüd ka kui palju miine antud kordinaadi ümbruses
-- leidub. Võite eeldada, et konstruktori teine argument on alati vahemikus 0
-- kuni 8.
data Open = Open Coord Int
  deriving (Show, Read)

----
-- Mängu konfiguratsioon.
data GameConf = GameConf {
    height :: Int,
    width :: Int,
    nMines :: Int
  }
  deriving (Show, Read)

----
-- Mängu tulemus.
-- Loodetavasti viike on vähe.
data GameOutcome
  = Win
  | Tie
  | Loss
  deriving (Show, Read)

----
-- Turnriiri lõpu põhjendus.
-- Hetkel salvestame siin ainult saavutatud positsiooni. Selle andmetüübi
-- väljad võivad tulevikus muutuda.
data Reason = TournamentOver {
    position :: (Int, Int)
  }
  deriving (Show, Read)

----
-- Sõnumid mida klient võib serverile saata.
-- Vähemalt üks konstruktori "Clicks" argument peab mittetühi olema.
data ClientMessage
  = Hello Name
  | Games [Board] -- Games <viiskümmend mängu>
  | Clicks [Coord] [Coord] -- Clicks <avada> <ohtlikuks määrata>
  deriving (Show, Read)

{-
 - Kliendi poolt saadetavate sõnumite järjekord peaks välja
 - nägema järgmine:
 -
 - Hello -> (Games -> Clicks*) x 50
 -}

----
-- Sõnumida mida server võib kliendile saata.
data ServerMessage
  = NewGame Name GameConf -- mäng uue vastasega
  | NewField -- uus väljak
  | Opened [Open]
  | FieldDone Double -- väljak on kaotatud või võidetud
  | GameOver GameOutcome
  | End Reason
  deriving (Show, Read)

{-
 - Eeldusel, et kliendid järgivad protokolli saadab server
 - fikseeritud kliendile järgmiselt sõnumeid:
 -
 - (NewGame -> (NewField -> Opened* -> FieldDone) x 50 -> GameOver)* -> End
 -}
