module Main (main) where

----
-- protokoll on imporditud qualified võtmesõnaga, et vältida
-- nimekonflikte
import qualified Prot as Prot
import Field

----
-- süsteemi moodulid mida kasutame
import System.Random
import System.IO 
import Control.Exception (finally)
import Network

type Rand a = StdGen -> (a, StdGen)

----
-- Port mida server kasutab.
port :: PortID
port = PortNumber 49267

----
-- Serveri aadress.
host :: HostName
host = "127.0.0.1"

----
-- Loob 50 paarikaupa erinevat väljakut.
mkGames :: Prot.GameConf -> Rand [Prot.Board]
mkGames = undefined

mkField :: Prot.GameConf -> Field
mkField = undefined

----
-- Väljaku uuendamine.
updateField :: [Prot.Open] -> [Prot.Coord] -> Field -> Field
updateField = undefined

----
-- Väljaku lahendamine.
-- NB! Serveri koordinaadid on kujul (y, x), kusjuures (1, 1) on ülemine vasak
-- ning (height, width) parem alumine nurk.
solveField :: Field -> Rand ([Prot.Coord], [Prot.Coord])
solveField = undefined

----
-- Protseduur, mis realiseerib serveriga suhtluse.
game :: Handle -> IO ()
game handle = let
    -- peamine tsükkel
    mainLoop cfg = do
      msg <- recv
      case msg of
        Prot.NewGame name newCfg -> do
          games <- getStdRandom (mkGames newCfg)
          send (Prot.Games games)
          mainLoop newCfg
	Prot.NewField -> roundLoop (mkField cfg) cfg
        Prot.GameOver status -> do
          putStrLn $ "Game over: " ++ show status
          mainLoop (error "Server bug!")
        Prot.End stat -> print stat

    -- tsükkel ühe väljaku lahendamiseks
    roundLoop field cfg = do
      (cs, fs) <- getStdRandom (solveField field)
      send (Prot.Clicks cs fs)
      msg <- recv
      case msg of
        Prot.Opened xs -> roundLoop (updateField xs fs field) cfg
        Prot.FieldDone score -> do
          putStrLn $ "Solved " ++ show (score*100.0) ++ "% of field."
          mainLoop cfg
        _ -> error "Server bug!"

    -- serverile sõnumi saatmine
    send msg = do
      hPutStrLn handle (show msg)
      hFlush handle

    -- serverilt sõnumi küsimine
    recv = do
      str <- hGetLine handle
      return $ read str
  in do
    send (Prot.Hello "<name>") -- siia tuleks mõistlikum nimi panna
    mainLoop (error "Server bug!")

main :: IO ()
main = withSocketsDo $ do
  handle <- connectTo host port -- loome ühenduse
  hSetBuffering handle LineBuffering
  (game handle `finally` hClose handle)
